@@ -8,6 +8,7 @@ import time
 import types
 import os
 import sys
+import zipfile
 
 from pysnooper.utils import truncate
 import pytest
@@ -25,7 +26,7 @@ from . import mini_toolbox
 def test_string_io():
     string_io = io.StringIO()
 
-    @pysnooper.snoop(string_io)
+    @pysnooper.snoop(string_io, color=False)
     def my_function(foo):
         x = 7
         y = 8
@@ -53,7 +54,7 @@ def test_string_io():
 
 
 def test_relative_time():
-    snoop = pysnooper.snoop(relative_time=True)
+    snoop = pysnooper.snoop(relative_time=True, color=False)
 
     def foo(x):
         if x == 0:
@@ -186,7 +187,7 @@ def test_relative_time():
 
 def test_thread_info():
 
-    @pysnooper.snoop(thread_info=True)
+    @pysnooper.snoop(thread_info=True, color=False)
     def my_function(foo):
         x = 7
         y = 8
@@ -217,7 +218,7 @@ def test_thread_info():
 
 def test_multi_thread_info():
 
-    @pysnooper.snoop(thread_info=True)
+    @pysnooper.snoop(thread_info=True, color=False)
     def my_function(foo):
         x = 7
         y = 8
@@ -306,7 +307,7 @@ def test_callable(normalize):
     def write(msg):
         string_io.write(msg)
 
-    @pysnooper.snoop(write, normalize=normalize)
+    @pysnooper.snoop(write, normalize=normalize, color=False)
     def my_function(foo):
         x = 7
         y = 8
@@ -347,7 +348,7 @@ def test_watch(normalize):
             'foo.x',
             'io.__name__',
             'len(foo.__dict__["x"] * "abc")',
-    ), normalize=normalize)
+    ), normalize=normalize, color=False)
     def my_function():
         foo = Foo()
         for i in range(2):
@@ -395,7 +396,8 @@ def test_watch_explode(normalize):
             self.x = x
             self.y = y
 
-    @pysnooper.snoop(watch_explode=('_d', '_point', 'lst + []'), normalize=normalize)
+    @pysnooper.snoop(watch_explode=('_d', '_point', 'lst + []'), normalize=normalize,
+                     color=False)
     def my_function():
         _d = {'a': 1, 'b': 2, 'c': 'ignore'}
         _point = Foo(x=3, y=4)
@@ -454,7 +456,7 @@ def test_variables_classes(normalize):
             pysnooper.Attrs('_d'),  # doesn't have attributes
             pysnooper.Attrs('_s'),
             pysnooper.Indices('_lst')[-3:],
-    ), normalize=normalize)
+    ), normalize=normalize, color=False)
     def my_function():
         _d = {'a': 1, 'b': 2, 'c': 'ignore'}
         _s = WithSlots()
@@ -501,7 +503,7 @@ def test_single_watch_no_comma(normalize):
         def square(self):
             self.x **= 2
 
-    @pysnooper.snoop(watch='foo', normalize=normalize)
+    @pysnooper.snoop(watch='foo', normalize=normalize, color=False)
     def my_function():
         foo = Foo()
         for i in range(2):
@@ -537,7 +539,7 @@ def test_single_watch_no_comma(normalize):
 
 @pytest.mark.parametrize("normalize", (True, False))
 def test_long_variable(normalize):
-    @pysnooper.snoop(normalize=normalize)
+    @pysnooper.snoop(normalize=normalize, color=False)
     def my_function():
         foo = list(range(1000))
         return foo
@@ -566,7 +568,7 @@ def test_long_variable(normalize):
 
 @pytest.mark.parametrize("normalize", (True, False))
 def test_long_variable_with_custom_max_variable_length(normalize):
-    @pysnooper.snoop(max_variable_length=200, normalize=normalize)
+    @pysnooper.snoop(max_variable_length=200, normalize=normalize, color=False)
     def my_function():
         foo = list(range(1000))
         return foo
@@ -595,7 +597,7 @@ def test_long_variable_with_custom_max_variable_length(normalize):
 
 @pytest.mark.parametrize("normalize", (True, False))
 def test_long_variable_with_infinite_max_variable_length(normalize):
-    @pysnooper.snoop(max_variable_length=None, normalize=normalize)
+    @pysnooper.snoop(max_variable_length=None, normalize=normalize, color=False)
     def my_function():
         foo = list(range(1000))
         return foo
@@ -628,7 +630,7 @@ def test_repr_exception(normalize):
         def __repr__(self):
             1 / 0
 
-    @pysnooper.snoop(normalize=normalize)
+    @pysnooper.snoop(normalize=normalize, color=False)
     def my_function():
         bad = Bad()
 
@@ -669,7 +671,7 @@ def test_depth(normalize):
         result2 = f3(x2)
         return result2
 
-    @pysnooper.snoop(string_io, depth=3, normalize=normalize)
+    @pysnooper.snoop(string_io, depth=3, normalize=normalize, color=False)
     def f1(x1):
         result1 = f2(x1)
         return result1
@@ -722,7 +724,8 @@ def test_method_and_prefix(normalize):
         def __init__(self):
             self.x = 2
 
-        @pysnooper.snoop(watch=('self.x',), prefix='ZZZ', normalize=normalize)
+        @pysnooper.snoop(watch=('self.x',), prefix='ZZZ', normalize=normalize,
+                         color=False)
         def square(self):
             foo = 7
             self.x **= 2
@@ -762,7 +765,7 @@ def test_file_output(normalize):
     with mini_toolbox.create_temp_folder(prefix='pysnooper') as folder:
         path = folder / 'foo.log'
 
-        @pysnooper.snoop(path, normalize=normalize)
+        @pysnooper.snoop(path, normalize=normalize, color=False)
         def my_function(_foo):
             x = 7
             y = 8
@@ -800,7 +803,7 @@ def test_confusing_decorator_lines(normalize):
 
     @empty_decorator
     @pysnooper.snoop(string_io, normalize=normalize,
-                     depth=2)  # Multi-line decorator for extra confusion!
+                     depth=2, color=False)
     @empty_decorator
     @empty_decorator
     def my_function(foo):
@@ -840,7 +843,7 @@ def test_confusing_decorator_lines(normalize):
 @pytest.mark.parametrize("normalize", (True, False))
 def test_lambda(normalize):
     string_io = io.StringIO()
-    my_function = pysnooper.snoop(string_io, normalize=normalize)(lambda x: x ** 2)
+    my_function = pysnooper.snoop(string_io, normalize=normalize, color=False)(lambda x: x ** 2)
     result = my_function(7)
     assert result == 49
     output = string_io.getvalue()
@@ -866,7 +869,7 @@ def test_unavailable_source():
         python_file_path = folder / ('%s.py' % (module_name,))
         content = textwrap.dedent(u'''
             import pysnooper
-            @pysnooper.snoop()
+            @pysnooper.snoop(color=False)
             def f(x):
                 return x
         ''')
@@ -898,7 +901,7 @@ def test_no_overwrite_by_default():
         path = folder / 'foo.log'
         with path.open('w') as output_file:
             output_file.write(u'lala')
-        @pysnooper.snoop(str(path))
+        @pysnooper.snoop(str(path), color=False)
         def my_function(foo):
             x = 7
             y = 8
@@ -932,7 +935,7 @@ def test_overwrite():
         path = folder / 'foo.log'
         with path.open('w') as output_file:
             output_file.write(u'lala')
-        @pysnooper.snoop(str(path), overwrite=True)
+        @pysnooper.snoop(str(path), overwrite=True, color=False)
         def my_function(foo):
             x = 7
             y = 8
@@ -975,7 +978,7 @@ def test_overwrite():
 def test_error_in_overwrite_argument():
     with mini_toolbox.create_temp_folder(prefix='pysnooper') as folder:
         with pytest.raises(Exception, match='can only be used when writing'):
-            @pysnooper.snoop(overwrite=True)
+            @pysnooper.snoop(overwrite=True, color=False)
             def my_function(foo):
                 x = 7
                 y = 8
@@ -1002,7 +1005,7 @@ def test_needs_parentheses():
 @pytest.mark.parametrize("normalize", (True, False))
 def test_with_block(normalize):
     # Testing that a single Tracer can handle many mixed uses
-    snoop = pysnooper.snoop(normalize=normalize)
+    snoop = pysnooper.snoop(normalize=normalize, color=False)
 
     def foo(x):
         if x == 0:
@@ -1151,7 +1154,7 @@ def test_with_block_depth(normalize):
 
     def f1(x1):
         str(3)
-        with pysnooper.snoop(string_io, depth=3, normalize=normalize):
+        with pysnooper.snoop(string_io, depth=3, normalize=normalize, color=False):
             result1 = f2(x1)
         return result1
 
@@ -1210,7 +1213,7 @@ def test_cellvars(normalize):
         return f3(a)
 
     def f1(a):
-        with pysnooper.snoop(string_io, depth=4, normalize=normalize):
+        with pysnooper.snoop(string_io, depth=4, normalize=normalize, color=False):
             result1 = f2(a)
         return result1
 
@@ -1275,7 +1278,7 @@ def test_var_order(normalize):
 
         five, six, seven = 5, 6, 7
 
-    with pysnooper.snoop(string_io, depth=2, normalize=normalize):
+    with pysnooper.snoop(string_io, depth=2, normalize=normalize, color=False):
         result = f(1, 2, 3, 4)
 
     output = string_io.getvalue()
@@ -1344,7 +1347,7 @@ def test_generator():
     original_tracer_active = lambda: (sys.gettrace() is original_tracer)
 
 
-    @pysnooper.snoop(string_io)
+    @pysnooper.snoop(string_io, color=False)
     def f(x1):
         assert not original_tracer_active()
         x2 = (yield x1)
@@ -1441,7 +1444,7 @@ def test_custom_repr(normalize):
         (large, print_list_size),
         (dict, print_dict),
         (evil_condition, lambda x: 'I am evil')),
-            normalize=normalize,)
+            normalize=normalize, color=False)
     def sum_to_x(x):
         l = list(range(x))
         a = {'1': 1, '2': 2}
@@ -1473,7 +1476,8 @@ def test_custom_repr(normalize):
 def test_custom_repr_single(normalize):
     string_io = io.StringIO()
 
-    @pysnooper.snoop(string_io, custom_repr=(list, lambda l: 'foofoo!'), normalize=normalize)
+    @pysnooper.snoop(string_io, custom_repr=(list, lambda l: 'foofoo!'),
+                     normalize=normalize, color=False)
     def sum_to_x(x):
         l = list(range(x))
         return 7
@@ -1507,7 +1511,7 @@ def test_disable():
         return x + y
 
     with mini_toolbox.TempValueSetter((pysnooper.tracer, 'DISABLED'), True):
-        tracer = pysnooper.snoop(string_io)
+        tracer = pysnooper.snoop(string_io, color=False)
         with tracer:
             result = my_function('baba')
         my_decorated_function = tracer(my_function)
@@ -1521,7 +1525,7 @@ def test_disable():
 def test_class(normalize):
     string_io = io.StringIO()
 
-    @pysnooper.snoop(string_io, normalize=normalize)
+    @pysnooper.snoop(string_io, normalize=normalize, color=False)
     class MyClass(object):
         def __init__(self):
             self.x = 7
@@ -1568,7 +1572,7 @@ def test_class_with_decorated_method(normalize):
             return result
         return wrapper
 
-    @pysnooper.snoop(string_io, normalize=normalize)
+    @pysnooper.snoop(string_io, normalize=normalize, color=False)
     class MyClass(object):
         def __init__(self):
             self.x = 7
@@ -1617,13 +1621,13 @@ def test_class_with_decorated_method_and_snoop_applied_to_method(normalize):
             return result
         return wrapper
 
-    @pysnooper.snoop(string_io, normalize=normalize)
+    @pysnooper.snoop(string_io, normalize=normalize, color=False)
     class MyClass(object):
         def __init__(self):
             self.x = 7
 
         @decorator
-        @pysnooper.snoop(string_io, normalize=normalize)
+        @pysnooper.snoop(string_io, normalize=normalize, color=False)
         def my_method(self, foo):
             y = 8
             return y + self.x
@@ -1671,7 +1675,7 @@ def test_class_with_decorated_method_and_snoop_applied_to_method(normalize):
 def test_class_with_property(normalize):
     string_io = io.StringIO()
 
-    @pysnooper.snoop(string_io, normalize=normalize)
+    @pysnooper.snoop(string_io, normalize=normalize, color=False)
     class MyClass(object):
         def __init__(self):
             self._x = 0
@@ -1759,7 +1763,7 @@ def test_snooping_on_class_does_not_cause_base_class_to_be_snooped(normalize):
         def method_on_base_class(self):
             self.method_on_base_class_was_called = True
 
-    @pysnooper.snoop(string_io, normalize=normalize)
+    @pysnooper.snoop(string_io, normalize=normalize, color=False)
     class MyClass(UnsnoopedBaseClass):
         def method_on_child_class(self):
             self.method_on_base_class()
@@ -1793,7 +1797,7 @@ def test_normalize():
         def __init__(self, a):
             self.a = a
 
-    @pysnooper.snoop(string_io, normalize=True)
+    @pysnooper.snoop(string_io, normalize=True, color=False)
     def add():
         a = A(19)
         b = A(22)
@@ -1830,7 +1834,7 @@ def test_normalize_prefix():
         def __init__(self, a):
             self.a = a
 
-    @pysnooper.snoop(string_io, normalize=True, prefix=_prefix)
+    @pysnooper.snoop(string_io, normalize=True, prefix=_prefix, color=False)
     def add():
         a = A(19)
         b = A(22)
@@ -1866,7 +1870,7 @@ def test_normalize_thread_info():
         def __init__(self, a):
             self.a = a
 
-    @pysnooper.snoop(string_io, normalize=True, thread_info=True)
+    @pysnooper.snoop(string_io, normalize=True, thread_info=True, color=False)
     def add():
         a = A(19)
         b = A(22)
@@ -1879,7 +1883,7 @@ def test_normalize_thread_info():
 
 def test_exception():
     string_io = io.StringIO()
-    @pysnooper.snoop(string_io)
+    @pysnooper.snoop(string_io, color=False)
     def f():
         x = 8
         raise MemoryError
@@ -1905,9 +1909,159 @@ def test_exception():
 
 
 def test_exception_on_entry():
-    @pysnooper.snoop()
+    @pysnooper.snoop(color=False)
     def f(x):
         pass
 
     with pytest.raises(TypeError):
         f()
+
+
+def test_valid_zipfile():
+    with mini_toolbox.create_temp_folder(prefix='pysnooper') as folder, \
+                                    mini_toolbox.TempSysPathAdder(str(folder)):
+        module_name = 'my_valid_zip_module'
+        zip_name = 'valid.zip'
+        zip_base_path = mini_toolbox.pathlib.Path('ansible/modules')
+        python_file_path = folder / zip_name / zip_base_path / ('%s.py' % (module_name))
+        os.makedirs(str(folder / zip_name / zip_base_path))
+        try:
+            sys.path.insert(0, str(folder / zip_name / zip_base_path))
+            content = textwrap.dedent(u'''
+                import pysnooper
+                @pysnooper.snoop(color=False)
+                def f(x):
+                    return x
+            ''')
+
+            python_file_path.write_text(content)
+
+            module = __import__(module_name)
+
+            with zipfile.ZipFile(str(folder / 'foo_bar.zip'), 'w') as myZipFile:
+                myZipFile.write(str(folder / zip_name / zip_base_path / ('%s.py' % (module_name))), \
+                                '%s/%s.py' % (zip_base_path, module_name,), \
+                                zipfile.ZIP_DEFLATED)
+
+            python_file_path.unlink()
+            folder.joinpath(zip_name).rename(folder.joinpath('%s.delete' % (zip_name)))
+            folder.joinpath('foo_bar.zip').rename(folder.joinpath(zip_name))
+
+            with mini_toolbox.OutputCapturer(stdout=False,
+                                             stderr=True) as output_capturer:
+                result = getattr(module, 'f')(7)
+            assert result == 7
+            output = output_capturer.output
+
+            assert_output(
+                output,
+                (
+                    SourcePathEntry(),
+                    VariableEntry(stage='starting'),
+                    CallEntry('def f(x):'),
+                    LineEntry('return x'),
+                    ReturnEntry('return x'),
+                    ReturnValueEntry('7'),
+                    ElapsedTimeEntry(),
+                )
+            )
+        finally:
+            sys.path.remove(str(folder / zip_name / zip_base_path))
+
+
+def test_invalid_zipfile():
+    with mini_toolbox.create_temp_folder(prefix='pysnooper') as folder, \
+                                    mini_toolbox.TempSysPathAdder(str(folder)):
+        module_name = 'my_invalid_zip_module'
+        zip_name = 'invalid.zip'
+        zip_base_path = mini_toolbox.pathlib.Path('invalid/modules/path')
+        python_file_path = folder / zip_name / zip_base_path / ('%s.py' % (module_name))
+        os.makedirs(str(folder / zip_name / zip_base_path))
+        try:
+            sys.path.insert(0, str(folder / zip_name / zip_base_path))
+            content = textwrap.dedent(u'''
+                import pysnooper
+                @pysnooper.snoop(color=False)
+                def f(x):
+                    return x
+            ''')
+            python_file_path.write_text(content)
+
+            module = __import__(module_name)
+
+            with zipfile.ZipFile(str(folder / 'foo_bar.zip'), 'w') as myZipFile:
+                myZipFile.write(str(folder / zip_name / zip_base_path / ('%s.py' % (module_name))), \
+                                str(zip_base_path / ('%s.py' % (module_name,))), \
+                                zipfile.ZIP_DEFLATED)
+
+            python_file_path.unlink()
+            folder.joinpath(zip_name).rename(folder.joinpath('%s.delete' % (zip_name)))
+            folder.joinpath('foo_bar.zip').rename(folder.joinpath(zip_name))
+
+            with mini_toolbox.OutputCapturer(stdout=False,
+                                         stderr=True) as output_capturer:
+                result = getattr(module, 'f')(7)
+            assert result == 7
+            output = output_capturer.output
+
+            assert_output(
+                output,
+                (
+                    SourcePathEntry(),
+                    VariableEntry(stage='starting'),
+                    CallEntry('SOURCE IS UNAVAILABLE'),
+                    LineEntry('SOURCE IS UNAVAILABLE'),
+                    ReturnEntry('SOURCE IS UNAVAILABLE'),
+                    ReturnValueEntry('7'),
+                    ElapsedTimeEntry(),
+                )
+            )
+        finally:
+            sys.path.remove(str(folder / zip_name / zip_base_path))
+
+
+def test_valid_damaged_zipfile():
+    with mini_toolbox.create_temp_folder(prefix='pysnooper') as folder, \
+                                    mini_toolbox.TempSysPathAdder(str(folder)):
+        module_name = 'my_damaged_module'
+        zip_name = 'damaged.zip'
+        zip_base_path = mini_toolbox.pathlib.Path('ansible/modules')
+        python_file_path = folder / zip_name / zip_base_path / ('%s.py' % (module_name))
+        os.makedirs(str(folder / zip_name / zip_base_path))
+        try:
+            sys.path.insert(0, str(folder / zip_name / zip_base_path))
+            content = textwrap.dedent(u'''
+                import pysnooper
+                @pysnooper.snoop(color=False)
+                def f(x):
+                    return x
+            ''')
+            python_file_path.write_text(content)
+
+            module = __import__(module_name)
+
+            python_file_path.unlink()
+            folder.joinpath(zip_name).rename(folder.joinpath('%s.delete' % (zip_name)))
+
+            folder.joinpath(zip_name).write_text(u'I am not a zip file')
+
+            with mini_toolbox.OutputCapturer(stdout=False,
+                                         stderr=True) as output_capturer:
+                result = getattr(module, 'f')(7)
+            assert result == 7
+            output = output_capturer.output
+
+            assert_output(
+                output,
+                (
+                    SourcePathEntry(),
+                    VariableEntry(stage='starting'),
+                    CallEntry('SOURCE IS UNAVAILABLE'),
+                    LineEntry('SOURCE IS UNAVAILABLE'),
+                    ReturnEntry('SOURCE IS UNAVAILABLE'),
+                    ReturnValueEntry('7'),
+                    ElapsedTimeEntry(),
+                )
+            )
+        finally:
+            sys.path.remove(str(folder / zip_name / zip_base_path))
